@@ -1,12 +1,14 @@
 package App::fiatx;
 
-our $DATE = '2018-05-10'; # DATE
-our $VERSION = '0.002'; # VERSION
+our $DATE = '2018-06-19'; # DATE
+our $VERSION = '0.003'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 
+use Finance::Currency::FiatX;
+
 our %SPEC;
 
 our %args_db = (
@@ -27,44 +29,6 @@ our %args_db = (
     },
 );
 
-our %args_convert = (
-    amount => {
-        schema => 'num*',
-        default => 1,
-    },
-    from => {
-        schema => 'currency::code*',
-        req => 1,
-    },
-    to => {
-        schema => 'currency::code*',
-        req => 1,
-    },
-    type => {
-        summary => 'Which rate is wanted? e.g. sell, buy',
-        schema => 'str*',
-        default => 'sell', # because we want to buy
-    },
-);
-
-our %args_caching = (
-    max_age_cache => {
-        summary => 'Above this age (in seconds), '.
-            'we retrieve rate from remote source again',
-        schema => 'posint*',
-        default => 4*3600,
-        cmdline_aliases => {
-            no_cache => {is_flag=>1, code=>sub {$_[0]{max_age_cache} = 0}, summary=>'Alias for --max-age-cache=0'},
-        },
-    },
-    max_age_current => {
-        summary => 'Above this age (in seconds), '.
-            'we no longer consider the rate to be "current" but "historical"',
-        schema => 'posint*',
-        default => 24*3600,
-    },
-);
-
 sub _connect {
     my $args = shift;
 
@@ -89,32 +53,75 @@ sub _supply {
     %res;
 }
 
-$SPEC{convert} = {
+$SPEC{sources} = {
     v => 1.1,
-    summary => 'Convert two currencies using current rate',
+    summary => 'List available sources',
     args => {
-        %args_db,
-        %args_caching,
-        %args_convert,
     },
 };
-sub convert {
-    require Finance::Currency::FiatX;
+sub sources {
+    require PERLANCAR::Module::List;
 
+    my @res;
+    my $mods = PERLANCAR::Module::List::list_modules(
+        'Finance::Currency::FiatX::Source::', {list_modules=>1});
+    unless (keys %$mods) {
+        return [412, "No source modules available"];
+    }
+    for my $src (sort keys %$mods) {
+        $src =~ s/^Finance::Currency::FiatX::Source:://;
+        push @res, $src;
+    }
+
+    [200, "OK", \@res];
+}
+
+$SPEC{spot_rate} = {
+    v => 1.1,
+    summary => 'Get spot (latest) rate',
+    args => {
+        %args_db,
+        %Finance::Currency::FiatX::args_caching,
+        %Finance::Currency::FiatX::args_spot_rate,
+    },
+};
+sub spot_rate {
     my %args = @_;
 
     my $dbh = _connect(\%args);
 
-    Finance::Currency::FiatX::convert_fiat_currency(
+    Finance::Currency::FiatX::get_spot_rate(
         dbh => $dbh,
 
-        _supply(\%args, \%args_caching),
-        _supply(\%args, \%args_convert),
+        _supply(\%args, \%Finance::Currency::FiatX::args_caching),
+        _supply(\%args, \%Finance::Currency::FiatX::args_spot_rate),
+    );
+}
+
+$SPEC{all_spot_rates} = {
+    v => 1.1,
+    summary => 'Get all spot (latest) rates from a source',
+    args => {
+        %args_db,
+        %Finance::Currency::FiatX::args_caching,
+        %Finance::Currency::FiatX::arg_req0_source,
+    },
+};
+sub all_spot_rates {
+    my %args = @_;
+
+    my $dbh = _connect(\%args);
+
+    Finance::Currency::FiatX::get_all_spot_rates(
+        dbh => $dbh,
+
+        _supply(\%args, \%Finance::Currency::FiatX::args_caching),
+        _supply(\%args, \%Finance::Currency::FiatX::arg_req0_source),
     );
 }
 
 1;
-# ABSTRACT: Convert two currencies using current rate
+# ABSTRACT: Get all spot (latest) rates from a source
 
 __END__
 
@@ -124,11 +131,11 @@ __END__
 
 =head1 NAME
 
-App::fiatx - Convert two currencies using current rate
+App::fiatx - Get all spot (latest) rates from a source
 
 =head1 VERSION
 
-This document describes version 0.002 of App::fiatx (from Perl distribution App-fiatx), released on 2018-05-10.
+This document describes version 0.003 of App::fiatx (from Perl distribution App-fiatx), released on 2018-06-19.
 
 =head1 SYNOPSIS
 
@@ -137,13 +144,13 @@ See the included script L<fiatx>.
 =head1 FUNCTIONS
 
 
-=head2 convert
+=head2 all_spot_rates
 
 Usage:
 
- convert(%args) -> [status, msg, result, meta]
+ all_spot_rates(%args) -> [status, msg, result, meta]
 
-Convert two currencies using current rate.
+Get all spot (latest) rates from a source.
 
 This function is not exported.
 
@@ -151,7 +158,81 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<amount> => I<num> (default: 1)
+=item * B<db_name>* => I<str>
+
+=item * B<db_password> => I<str>
+
+=item * B<db_username> => I<str>
+
+=item * B<max_age_cache> => I<posint> (default: 14400)
+
+Above this age (in seconds), we retrieve rate from remote source again.
+
+=item * B<source>* => I<str> (default: ":any")
+
+Ask for a specific remote source.
+
+If you want a specific remote source, you can specify it here. The default is
+':any' which is to pick the first source that returns a recent enough current
+rate.
+
+Other special values: C<:highest> to return highest rate of all sources,
+C<:lowest> to return lowest rate of all sources, ':newest' to return rate from
+source with the newest last update time, ':oldest' to return rate from source
+with the oldest last update time, ':average' to return arithmetic average of all
+sources.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 sources
+
+Usage:
+
+ sources() -> [status, msg, result, meta]
+
+List available sources.
+
+This function is not exported.
+
+No arguments.
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 spot_rate
+
+Usage:
+
+ spot_rate(%args) -> [status, msg, result, meta]
+
+Get spot (latest) rate.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
 
 =item * B<db_name>* => I<str>
 
@@ -165,9 +246,19 @@ Arguments ('*' denotes required arguments):
 
 Above this age (in seconds), we retrieve rate from remote source again.
 
-=item * B<max_age_current> => I<posint> (default: 86400)
+=item * B<source> => I<str> (default: ":any")
 
-Above this age (in seconds), we no longer consider the rate to be "current" but "historical".
+Ask for a specific remote source.
+
+If you want a specific remote source, you can specify it here. The default is
+':any' which is to pick the first source that returns a recent enough current
+rate.
+
+Other special values: C<:highest> to return highest rate of all sources,
+C<:lowest> to return lowest rate of all sources, ':newest' to return rate from
+source with the newest last update time, ':oldest' to return rate from source
+with the oldest last update time, ':average' to return arithmetic average of all
+sources.
 
 =item * B<to>* => I<currency::code>
 
